#include "common.h"
#include "device_change.h"
#include "download.h"

#define BUFFER_SIZE 1024

char *DEVICE_NAME_FILE="/home/orangepi/car/master/Deviceld.txt";

// 检查并创建日志目录
static int create_devicename_directory_if_not_exists(const char *path) {
    char dir_path[256] = {0};
    const char *p = path;
    char *q = dir_path;
    
    // 提取目录部分
    while (*p != '\0') {
        *q = *p;
        if (*p == '/' && *(p+1) != '\0') {
            // 检查并创建每一级目录
            if (access(dir_path, F_OK) != 0) {
                if (mkdir(dir_path, 0755) != 0) {
                    fprintf(stderr, "Failed to create directory: %s\n", dir_path);
                    my_zlog_error("创建目录失败");
                    return -1;
                }
            }
        }
        p++;
        q++;
    }
    
    return 0;
}

// 复制文件函数
int copy_file(const char *source, const char *destination) {
    FILE *src_file = fopen(source, "rb");
    if (src_file == NULL) {
        my_zlog_warn("无法打开源文件");
        return -1;
    }

    FILE *dst_file = fopen(destination, "wb");
    if (dst_file == NULL) {
        my_zlog_warn("无法打开目标文件");
        fclose(src_file);
        return -1;
    }

    char buffer[BUFFER_SIZE];
    size_t bytes_read;
    while ((bytes_read = fread(buffer, 1, BUFFER_SIZE, src_file)) > 0) {
        if (fwrite(buffer, 1, bytes_read, dst_file) != bytes_read) {
            my_zlog_warn("写入目标文件时出错");
            fclose(src_file);
            fclose(dst_file);
            return -1;
        }
    }

    fclose(src_file);
    fclose(dst_file);
    return 0;
}

// 改名并移动文件函数
int rename_and_move_file(const char *source_file, const char *new_name, const char *destination_dir) {
    char destination_file[BUFFER_SIZE];
    // 构建目标文件的完整路径
    snprintf(destination_file, sizeof(destination_file), "%s/%s", destination_dir, new_name);

    // 尝试使用 rename 函数移动文件
    if (rename(source_file, destination_file) == 0) {
        my_zlog_debug("文件已成功改名并移动到 %s\n", destination_file);
        return 0;
    }

    // 如果 rename 失败，可能是跨文件系统，尝试复制文件
    if (errno == EXDEV) {
        if (copy_file(source_file, destination_file) == 0) {
            // 复制成功后删除原文件
            if (remove(source_file) == 0) {
                my_zlog_debug("文件已成功改名并移动到 %s\n", destination_file);
                return 0;
            } else {
                my_zlog_warn("删除原文件时出错");
                return -1;
            }
        } else {
            my_zlog_warn("复制文件时出错");
            return -1;
        }
    }

    my_zlog_warn("移动文件时出错");
    return -1;
}

// 从文件名中提取时间戳
time_t extract_timestamp(const char *filename) {
    char timestamp_str[20];
    const char *dot = strrchr(filename, '.');
    if (dot != NULL) {
        strncpy(timestamp_str, dot + 1, sizeof(timestamp_str) - 1);
        timestamp_str[sizeof(timestamp_str) - 1] = '\0';

        struct tm tm = {0};
        if (strptime(timestamp_str, "%Y%m%d%H%M", &tm) != NULL) {
            return mktime(&tm);
        }
    }
    return 0;
}

// 查找最新的文件
const char *find_latest_file(const char *dir_path) {
    DIR *dir = opendir(dir_path);
    if (dir == NULL) {
        perror("无法打开目录");
        return NULL;
    }

    struct dirent *entry;
    time_t latest_time = 0;
    const char *latest_filename = NULL;

    while ((entry = readdir(dir)) != NULL) {
        if (entry->d_type == DT_REG) {
            time_t timestamp = extract_timestamp(entry->d_name);
            if (timestamp > latest_time) {
                latest_time = timestamp;
                if (latest_filename != NULL) {
                    free((char *)latest_filename);
                }
                latest_filename = strdup(entry->d_name);
            }
        }
    }

    closedir(dir);
    return latest_filename;
}

int device_changename_back() {//备份文件
    if(file_exists(DEVICE_NAME_FILE) == 0) {//先检测文件是否存在
        if(create_devicename_directory_if_not_exists(DEVICE_NAME_DIR) != 0) {
            my_zlog_error("创建文件目录失败");
            return -1;
        }

    char *DEVICE_NAME_NEW = sprintf(DEVICE_NAME_FILE,".%s",);

    if(rename_and_move_file(DEVICE_NAME_FILE,DEVICE_NAME_NEW,DEVICE_NAME_DIR) !=0 ){
        my_zlog_warn("移动文件失败");
    }

    }
    return 0;
}

char *device_changename_back_fail() {//改设备号密码失败回退版本,需要释放
    char *device_file=find_latest_file(DEVICE_NAME_DIR);
    const char *device_file_dir= "/home/orangepi/car/master/deviceback" ;
    // 计算拼接后字符串的长度
    size_t size = strlen(device_file_dir) + strlen(device_file) + 1 + 1; // 加上'/'和字符串结束符'\0'
    char *full_path = (char *)malloc(size);
    snprintf(full_path, size, "%s/%s", device_file_dir, device_file);
    my_zlog_debug("拼接后的完整路径: %s\n", full_path);
    free(device_file);
    return full_path;
}